import java.io.File;
import java.util.Locale;
import java.util.Scanner;
import javax.swing.JFileChooser;

/**
 * Liest den Inhalt der Schlange aus einer Datei.
 * 
 * @author Bernd Gramlich
 * @version 2012-11-18
 */
public class Datenleser
{
    private JFileChooser dateiAuswahl;
    
    private static Datenleser datenleserSingleton;

    /**
     * Konstruktor.
     * Erzeuge ein Dateiauswahlfenster, das auf das augenblickliche
     * Verzeichnis des Nutzers zeigt.
     * 
     * Der Konstruktor ist private, weil er von auen
     * nicht aufgerufen werden soll. Stattdessen soll
     * mit gibDatenleser () auf den Datenleser
     * zugegriffen werden.
     */
    private Datenleser ()
    {
        dateiAuswahl = new JFileChooser (System.getProperty ("user.dir"));
    }
    
    /**
     * Gib den einzigen verfgbaren Datenleser zurck
     * (also den "datenleserSingleton"). Wenn der
     * Datenleser noch nicht erzeugt ist, erzeuge ihn.
     */
    public static Datenleser gibDatenleser ()
    {
        if (null == datenleserSingleton)
        {
            datenleserSingleton = new Datenleser ();
        }
        
        return datenleserSingleton;
    }

    /**
     * Lies die vom Benutzer gewnschte Vokabeldatei in
     * die Schlange ein. Die Schlange wird dabei ergnzt.
     * Man kann also mehrere Dateien zusammen in die
     * Schlange laden.
     */
    public void lies ()
    {
        /*
         * ffne eine Dateiauswahlfenster zum Einlesen der
         * Vokabeln. Falls der Benutzer eine Datei ausgewhlt
         * hat ...
         */
        if (dateiAuswahl.showOpenDialog (null) == JFileChooser.APPROVE_OPTION)
        { 
            /*
             * ... ermittle die Quelldatei.
             */
            File quelldatei = dateiAuswahl.getSelectedFile ();
            
            try
            {
                /*
                 * ffne einen deutschsprachigen Leser fr die Datei.
                 */
                Scanner leser = new Scanner (quelldatei);
                leser.useLocale (Locale.GERMAN);
                
                /*
                 * Zhle die gelesenen Vokabeln und Vokabelbestandteile
                 * mit.
                 */
                int vokabelzahl = 0;
                int bestandteilezahl;
                
                /*
                 * Lies alle Vokabeln der Datei Stck fr Stck ein.
                 */
                do
                {
                    /*
                     * Bisher sind noch keine Bestandteile der neuen
                     * Vokabel geladen.
                     */
                    Vokabel neueVokabel = new Vokabel ("", "");
                    bestandteilezahl = 0;
                    
                    /*
                     * Versuche, nacheinander die vier Bestandteile
                     * fremdsprachige Bedeutung, deutsche Bedeutung,
                     * Punktzahl und Wissensstand der neuen Vokabel
                     * zu laden. Ersetze dabei Unterstriche durch
                     * Leerzeichen.
                     */
                    if (leser.hasNext ())
                    {
                        bestandteilezahl++;
                        neueVokabel.setzeFremd (leser.next ().replace ("_", " "));
                    }
                    
                    if (leser.hasNext ())
                    {
                        bestandteilezahl++;
                        neueVokabel.setzeDeutsch (leser.next ().replace ("_", " "));
                    }
                    
                    if (leser.hasNextDouble ())
                    {
                        bestandteilezahl++;
                        neueVokabel.setzePunkte (leser.nextDouble ());
                    }
                    
                    if (leser.hasNextDouble ())
                    {
                        bestandteilezahl++;
                        neueVokabel.setzeWissen (leser.nextDouble ());
                    }
                    
                    /*
                     * Die Punktzahl wird spter beim Einfgen um
                     * den Wissensstand erhht. Das muss hier korrigiert
                     * werden, damit nach dem Einfgen dieselbe Punktzahl
                     * vorliegt wie vor dem letzten Abspeichern.
                     */
                    neueVokabel.senkePunkte ();
                    
                    /*
                     * Wenn alle vier Bestandteile erfolgreich geladen
                     * sind, fge die Vokabel in die Kartei ein. Versuche
                     * anschlieend, noch eine Vokabel zu laden.
                     * 
                     * Wenn die vier Bestandteile nicht vorhanden sind,
                     * hre auf.
                     */
                    if (4 == bestandteilezahl)
                    {
                        vokabelzahl++;
                        Schlange.gibSchlange ().fuegePassendEin (neueVokabel);
                    }
                }
                while (4 == bestandteilezahl);
                
                /*
                 * Melde, wie viele Vokabeln geladen wurden und wie viele
                 * jetzt in der Kartei sind.
                 */
                System.out.println (vokabelzahl + " Vokabeln nachgeladen.");
                System.out.println ("Insgesamt " + Schlange.gibSchlange ().gibLaenge ()
                    + " Vokabeln in der Kartei.");
            }
            catch (Exception fehler)
            {
            }
        }
    }
}
