import java.io.File;
import java.io.PrintWriter;
import javax.swing.JFileChooser;

/**
 * Schreibt den Inhalt der Schlange in eine Datei.
 * 
 * @author Bernd Gramlich
 * @version 2012-11-17
 */
public class Datenschreiber
{
    private JFileChooser dateiAuswahl;
    
    private static Datenschreiber datenschreiberSingleton;

    /**
     * Konstruktor.
     * Erzeuge ein Dateiauswahlfenster, das auf das augenblickliche
     * Verzeichnis des Nutzers zeigt.
     * 
     * Der Konstruktor ist private, weil er von auen
     * nicht aufgerufen werden soll. Stattdessen soll
     * mit gibDatenschreiber () auf den Datenschreiber
     * zugegriffen werden.
     */
    private Datenschreiber ()
    {
        dateiAuswahl = new JFileChooser (System.getProperty ("user.dir"));
    }
    
    /**
     * Gib den einzigen verfgbaren Datenschreiber zurck
     * (also den "datenschreiberSingleton"). Wenn der
     * Datenschreiber noch nicht erzeugt ist, erzeuge ihn.
     */
    public static Datenschreiber gibDatenschreiber ()
    {
        if (null == datenschreiberSingleton)
        {
            datenschreiberSingleton = new Datenschreiber ();
        }
        
        return datenschreiberSingleton;
    }

    /**
     * Schreibe den Inhalt der Vokabelschlange in die vom
     * Benutzer gewnschte Datei. Der alte Dateiinhalt wird
     * dabei berschrieben. Das ist sinnvoll, weil stets der
     * neueste Lernstand gespeichert werden sollte.
     */
    public void schreibe ()
    {
        /*
         * ffne eine Dateiauswahlfenster zum Speichern der
         * Schlange. Falls der Benutzer eine Datei ausgewhlt
         * hat ...
         */
        if (dateiAuswahl.showSaveDialog (null) == JFileChooser.APPROVE_OPTION)
        { 
            /*
             * ... ermittle die Zieldatei.
             */
            File zieldatei = dateiAuswahl.getSelectedFile ();
            
            try
            {
                /*
                 * ffne einen Schreiber fr die Datei.
                 */
                PrintWriter schreiber = new PrintWriter (zieldatei);
                
                /*
                 * Schreibe den Inhalt der Schlange rekursiv in die
                 * Datei.
                 */
                Schlange.gibSchlange ().speichereAuf (schreiber);
                
                /*
                 * Beende das Schreiben.
                 */
                schreiber.flush ();
                schreiber.close ();
                
                /*
                 * Melde Erfolg.
                 */
                System.out.println (Schlange.gibSchlange ().gibLaenge ()
                    + " Vokabeln gespeichert!");
            }
            catch (Exception fehler)
            {
            }
        }
    }
}
