import java.util.Locale;
import java.io.PrintWriter;

/**
 * Ein Knoten einer Schlange aus Vokabeln.
 * 
 * @author Bernd Gramlich 
 * @version 2013-11-12
 */
public class Knoten extends Schlangenelement
{
    /*
     * Inhalt des Knotens ist eine Vokabel.
     */
    private Vokabel inhalt;

    /*
     * Nachfolger des Knotens ist ein Schlangenelement,
     * also entweder wieder ein Knoten oder der Abschluss
     * der Warteschlange.
     */
    private Schlangenelement nachfolger;

    /**
     * Konstruktor.
     * Erzeuge einen neuen Knoten an n-ter Position in der
     * Schlange mit Nachfolger "meinNachfolger" und Inhalt
     * "neueVokabel".
     */
    public Knoten (Schlangenelement meinNachfolger, int n,
        Vokabel neueVokabel)
    {
        nachfolger = meinNachfolger;
        inhalt     = neueVokabel;
    }

    /**
     * Nenne den Nachfolger des Knotens.
     */
    public Schlangenelement gibNachfolger ()
    {
        return nachfolger;
    }

    /**
     * Nenne den Inhalt des Knotens.
     */
    public Vokabel gibInhalt ()
    {
        return inhalt;
    }

    /**
     * Verschaffe dem Knoten einen neuen Nachfolger.
     */
    public void setzeNachfolger (Schlangenelement neuerNachfolger)
    {
        nachfolger = neuerNachfolger;
    }
    
    /**
     * Fge die "neueVokabel" in die nach aufsteigender
     * Punktzahl geordnete Vokabelschlange passend ein.
     * Ich selbst bin die n-te Vokabel.
     * 
     * Da ich der Abschluss bin, erzeuge ich einen neuen
     * Knoten mit mir als Nachfolger. Den Verweis auf mich
     * selbst erziele ich durch das Java-Schlsselwort "this".
     */
    public Schlangenelement fuegePassendEin (int n, Vokabel neueVokabel)
    {
        if (inhalt.gibPunkte () < neueVokabel.gibPunkte ())
        {
            /*
             * Wenn meine Punktzahl kleiner ist als die
             * der neuen Vokabel, dann muss die neue Vokabel
             * hinter mir eingefgt werden. Diese Aufgabe
             * berlasse ich meinem Nachfolger.
             * 
             * Mein Nachfolger ist das (n+1)-te Schlangenelement.
             */
            nachfolger = nachfolger.fuegePassendEin (n+1, neueVokabel);
            
            return this;
        }
        else
        {
            /*
             * Wenn die neue Vokabel hchstens so viele
             * Punkte hat wie ich, dann fge sie vor mir ein.
             * 
             * Die neue Vokabel ist dann das neue n-te
             * Schlangenelement.
             */
            return new Knoten (this, n, neueVokabel);
        }
    }

    /**
     * Nenne die Lnge der Schlange von mir selbst ab.
     */
    public int gibLaenge ()
    {
        return 1 + nachfolger.gibLaenge ();
    }
    
    /**
     * Nenne den geringsten Wissensstand innerhalb der
     * Schlange von mir selbst ab. (Das ist ein sehr
     * strenges Ma fr das erreichte Lernniveau. Je
     * grer das Ergebnis, um so hher der Erfolg.)
     */
    public double gibGeringstesWissen ()
    {
        double geringstesWissenHinterMir;
        
        geringstesWissenHinterMir = nachfolger.gibGeringstesWissen ();
        
        if (inhalt.gibWissen () < geringstesWissenHinterMir)
        {
            return inhalt.gibWissen ();
        }
        else
        {
            return geringstesWissenHinterMir;
        }
    }
    
    /**
     * Summiere innerhalb der Schlange von mir selbst
     * ab die Kehrwerte der Quadrate der Wissensstnde.
     * (Das ist ein grozgiges Ma fr das erreichte
     * Lernniveau. Je kleiner das Ergebnis, um so hher
     * der Erfolg.)
     */
    public double summiereReziprokquadratischesWissen ()
    {
        return 1 / (inhalt.gibWissen() * inhalt.gibWissen ())
            + nachfolger.summiereReziprokquadratischesWissen ();
    }
    
    /**
     * Verschiebe alle Punktzahlen innerhalb der Schlange
     * ab mir selbst um "summand".
     */
    public void verschiebePunkte (double summand)
    {
        inhalt.verschiebePunkte (summand);
        nachfolger.verschiebePunkte (summand);
    }
    
    /**
     * Speichere den Inhalt der Schlange ab mir selbst
     * auf einen Schreiber. Ersetze dabei Leerzeichen
     * durch Unterstriche, damit jede Zeichenkette auch
     * wieder als eine einzige Zeichenkette gelesen wird.
     * 
     * Um das Wiedereinlesen zu beschleunigen, wird die
     * Schlange in umgekehrter Reihenfolge gespeichert.
     */
    public void speichereAuf (PrintWriter schreiber)
    {
        nachfolger.speichereAuf (schreiber);
        
        schreiber.printf (Locale.GERMAN, "%s", inhalt.gibFremd ().replace (" ", "_"));
        schreiber.println ();
        schreiber.printf (Locale.GERMAN, "%s", inhalt.gibDeutsch ().replace (" ", "_"));
        schreiber.println ();
        schreiber.printf (Locale.GERMAN, "%f", inhalt.gibPunkte ());
        schreiber.println ();
        schreiber.printf (Locale.GERMAN, "%f", inhalt.gibWissen ());
        schreiber.println ();
    } 
}
