import java.io.PrintWriter;

/**
 * Eine Vokabelkartei.
 * Realisiert mit einer Schlange von Knoten.
 * 
 * @author Bernd Gramlich 
 * @version 2012-11-21
 */
public class Schlange
{
    private Schlangenelement erstesSchlangenelement;

    private static Schlange schlangenSingleton;

    /**
     * Konstruktor.
     * Erzeuge eine leere Schlange. Das ist also eine
     * Schlange, die nur aus einem Abschluss besteht.
     * 
     * Der Konstruktor ist private, weil er von auen
     * nicht aufgerufen werden soll. Stattdessen soll
     * mit gibSchlange () auf die Schlange zugegriffen
     * werden.
     */
    private Schlange()
    {
        erstesSchlangenelement = new Abschluss ();
    }

    /**
     * Gib die einzige verfgbare Schlange zurck (also den
     * "schlangenSingleton"). Wenn die Schlange noch nicht
     * erzeugt ist, erzeuge sie.
     */
    public static Schlange gibSchlange()
    {
        if (null == schlangenSingleton)
        {
            schlangenSingleton = new Schlange ();
        }

        return schlangenSingleton;
    }

    /**
     * Nchste Vokabel.
     * Nimm die vorderste Vokabel aus der Schlange und rcke auf.
     * Gib die Vokabel als Rckgabewert.
     * Falls die Schlange leer ist, gib null zurck.
     */
    public Vokabel naechsteVokabel ()
    {
        /*
         * Wir geben am Ende die erste Vokabel in der Schlange aus.
         */
        Vokabel ausgabe;

        ausgabe = erstesSchlangenelement.gibInhalt ();

        /*
         * Falls tatschlich eine Vokabel in der Schlange ist ...
         */
        if (null != ausgabe)
        {
            /*
             * ... vermindere die Punktzahl aller Vokabeln um die
             * Punktzahl der ersten Vokabel.
             */
            erstesSchlangenelement.verschiebePunkte (-ausgabe.gibPunkte ());
            ausgabe.verschiebePunkte (-ausgabe.gibPunkte ());

            /*
             * Der Nachfolger des ersten Schlangenelements wird
             * neues erstes Schlangenelement. 
             */
            erstesSchlangenelement = erstesSchlangenelement.gibNachfolger ();
        }

        return ausgabe;
    }

    /**
     * Neue Vokabel.
     * Fge gem den Parametern eine "neueVokabel" in die Schlange ein.
     */
    public void fuegePassendEin (Vokabel neueVokabel)
    {
        /*
         * Erhhe die Punktzahl der Vokabel um ihren Wissensstand.
         * Dadurch wird die Vokabel um so weiter hinten in die
         * Schlange eingeordnet, je besser man sie beherrscht.
         */
        neueVokabel.erhoehePunkte ();

        /*
         * Verndere die Punktzahl zufllig um bis zu +/-5% des
         * Wissenstandes, damit zur gleichen Zeit gelernte Vokabeln
         * in zuflliger Reihenfolge geprft werden.
         */
        neueVokabel.verschiebePunkte (
            (Math.random () - 0.5) * neueVokabel.gibWissen () / 10);
            
        /*
         * Fge die neue Vokabel in die Schlange ein.
         */
        erstesSchlangenelement =
            erstesSchlangenelement.fuegePassendEin (1, neueVokabel);
    }

    /**
     * Lnge.
     * Nenne die Lnge der Schlange.
     */
    public int gibLaenge ()
    {
        return erstesSchlangenelement.gibLaenge ();
    }

    /**
     * Nenne den geringsten Wissensstand innerhalb der
     * Schlange. (Das ist ein sehr strenges Ma fr das
     * erreichte Lernniveau. Je grer das Ergebnis, um
     * so hher der Erfolg.)
     */
    public double gibGeringstesWissen ()
    {
        return erstesSchlangenelement.gibGeringstesWissen ();
    }

    /**
     * Summiere die Kehrwerte der Quadrate der Wissensstnde
     * in der Schlange. (Das ist ein grozgiges Ma fr das
     * erreichte Lernniveau. Je kleiner das Ergebnis, um so
     * hher der Erfolg.)
     */
    public double summiereReziprokquadratischesWissen ()
    {
        return erstesSchlangenelement.summiereReziprokquadratischesWissen ();
    }

    /**
     * Speichere den Inhalt der Schlange auf einen Schreiber.
     */
    public void speichereAuf (PrintWriter schreiber)
    {
        erstesSchlangenelement.speichereAuf (schreiber);
    }   
}