import java.util.Scanner;

/**
 * Eine primitive Klasse zum ben von Vokabeln.
 * 
 * @author Bernd Gramlich 
 * @version 2012-11-21
 */
public class Trainer
{     
    private Scanner leser;
    
    private static Trainer trainerSingleton;

    /**
     * Konstruktor.
     * Erzeuge einen "leser", der die Standardeingabe (Tastatur)
     * abfragt.
     * 
     * Der Konstruktor ist private, weil er von auen
     * nicht aufgerufen werden soll. Stattdessen soll
     * mit gibTrainer () auf den Trainer zugegriffen
     * werden.
     */
    private Trainer ()
    {  
        leser = new Scanner (System.in);
    }
    
    /**
     * Gib den einzigen verfgbaren Trainer zurck (also
     * den "trainerSingleton"). Wenn der Trainer noch nicht
     * erzeugt ist, erzeuge ihn.
     */
    public static Trainer gibTrainer ()
    {
        if (null == trainerSingleton)
        {
            trainerSingleton = new Trainer ();
        }
        
        return trainerSingleton;
    }

    /**
     * be die nchsten Vokabeln, solange der Benutzer
     * Lust dazu hat.
     */
    public void uebe ()
    {
        Vokabel frage;
        String antwort;

        do
        {
            /*
             * Hole zur Ausfrage die nchste Vokabel aus der Schlange.
             */
            frage = Schlange.gibSchlange ().naechsteVokabel ();

            if (null == frage)
            {
                System.out.println ("Keine Vokabeln gespeichert.");
            }
            else
            {
                /*
                 * Nenne die fremdsprachige Bedeutung der Vokabel und
                 * warte auf einer Antwort des Benutzers.
                 */
                System.out.println ("Was heit   *** " + frage.gibFremd () + " *** ?");
                System.out.print ("> ");
                antwort = leser.nextLine ();
                
                /*
                 * Prfe, ob die Antwort mit der deutschen Bedeutung
                 * der Vokabel bereinstimmt.
                 */
                if (antwort.equals (frage.gibDeutsch ()))
                {
                    /*
                     * Wenn ja, erhhe den Wissenstand der Vokabel.
                     */
                    System.out.println ("Richtig.");
                    frage.erhoeheWissen ();
                }
                else
                {
                    /*
                     * Wenn nein, senke den Wissensstand der Vokabel.
                     */
                    System.out.println ("Falsch. Richtig ist: *** " + frage.gibDeutsch () + " ***");
                    frage.senkeWissen ();
                }
                
                /*
                 * Lege die Vokabel wieder in die Kartei zurck.
                 */
                Schlange.gibSchlange ().fuegePassendEin (frage);
            }
            
            /*
             * Nenne als Lernniveau die Summe der Kehrwerte der
             * Quadrate der Wissensstnde in der Schlange.
             * 
             * Ursprnglich stand hier als Lernniveau der
             * geringste Wissensstand einer Vokabel. Das war
             * aber frustrierend fr die Testpersonen, weil ihr
             * Lernerfolg dann immer nur an ihrer schwchsten
             * Vokabel gemessen wurde.
             */
            System.out.printf ("Lernniveau: %12.4f\n",
                Schlange.gibSchlange ().summiereReziprokquadratischesWissen ());
            
            /*
             * be weiter, wenn der Benutzer nur die Eingabetaste
             * drckt. Hre auf, wenn er zuvor noch eine andere
             * Taste drckt.
             */
            System.out.print("Weiter nur mit Eingabetaste, Ende mit anderer Taste zuvor. > ");
            antwort = leser.nextLine ();
        }
        while (antwort.equals (""));
    }
}
